#include <TFrameFormat.h>
#include <iostream>
#include <thread>
#include <atomic>
#include <condition_variable>
#include <vector>
#include <TSystem.h>

std::atomic<int> runningThreads(0);
std::mutex mtx;
std::condition_variable cv;

void thread(FrameFormat::TFrameChain &f, int globalID) {
   
    int fileID = f.GetFileID(globalID);
    int frameID = f.GetFrameID(globalID);

    FrameFormat::TFrame *ff = f.GetFrame(globalID);
    if(ff) ff->Load();

    runningThreads--;
    cv.notify_all();
}

int main() {

    unsigned int numCores = std::thread::hardware_concurrency();
    ROOT::EnableImplicitMT(numCores);
    std::cout << std::endl;
    std::cout << "CPU Information: " << ROOT::GetThreadPoolSize() << " core(s) available" << std::endl;

    FrameFormat::TFrameChain f("${PWD}/dataset/test-1.gwf ${PWD}/dataset/test-2.gwf", "d");
    f.SetVerbosity(FrameFormat::Verbosity::kDebug);

    std::vector<std::thread> threads;

    int N = f.GetNFrames();
    threads.reserve(N); // Preallocate space for threads

    for (int i = 0; i < N; ++i) {

        for (int cpu = 0; cpu < numCores; ++cpu) {
            threads.emplace_back(std::thread(&thread, std::ref(f), threads.size()));
            runningThreads++;
        }

        std::unique_lock<std::mutex> lock(mtx);
        cv.wait(lock, [&numCores] { return runningThreads < 1; });
        lock.unlock();

        for (int cpu = i * numCores; cpu < (i + 1) * numCores; ++cpu) {
            FrameFormat::TFrame *ff = f.GetFrame(cpu);
            if(ff) {

                ff->Dump();
                ff->Free();
            }
        }
    }

    
    for (auto &thread : threads) {
        thread.join();
    }

    return 0;
}
