#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <vector>  
#include "TFrameFormat.h" 

int main() 
{
    double GTimeS = 12345.;
    
    double sampleRate, phase, slope;
    long nData,i,j,nADC,sec;
    char name[100];
    
    /*----- add fasts (2kHz) ADC with data provided by a random generator ------------*/
    sampleRate = 10.;
    nData = 10;

    phase = 0;
    const char *comment = "test";
    const char *unitx = "Volts";
    int nBits = 16;

    std::vector<FrAdcData*> fastAdc(2, nullptr);
    for(i=0; i<fastAdc.size(); i++) {

        sprintf(name,"fastAdc%ld",i);
        slope =  20./65536;
        fastAdc[i] = FrAdcDataNewF(NULL, name, (char *)comment, 0, 0, nBits, 0., slope, (char *)unitx, sampleRate, nData);
        if(fastAdc[i] == NULL) return(0);
        
        for(j=0; j < nData; j++) {
            phase += .04;
            fastAdc[i]->data->dataS[j] = ((200.*rand())/RAND_MAX + 200.*cos(phase));
        }
    }

    /*----- add one process channel -----------*/
    FrProcData *proc = FrProcDataNew(NULL, "fastProc",sampleRate,nData,-32);
    FrProcDataAddHistory(proc, "fastProc = 2 *fastAdc",0);
    FrProcDataAddParam  (proc, "gain", 1.5);
 
    /*---- reserve space for slower (200 Hz) ADC's ------*/
    nData = nData / 10;
    sampleRate = sampleRate/10.;

    std::vector<FrAdcData*> adc(4, nullptr);
    for(i=0; i<adc.size(); i++) {

        sprintf(name,"Adc%ld",i);
        adc[i] = FrAdcDataNew(NULL, name,sampleRate,nData,16);
        for(j=0; j < adc[i]->data->nData; j++) {
            adc[i]->data->dataS[j] = (short) (rand());
        }
    }

    /*----- add one table -----------------------*/
    FrTable *table = FrTableNew("table_1","slow monitoring",5,2, FR_VECT_STRING, "channel", FR_VECT_4R,"value");        
    FrVect *col = FrTableGetCol(table,"value");
    for(j=0; j < col->nData; j++) {col->dataF[j] = .2*cos(.1*j);}


    /*---------------- reserve space for various type of data --*/
    FrAdcData *badAdc = FrAdcDataNew(NULL, "Bad_data_(data[0]=nan)",sampleRate,nData,-32);
    for(j=0; j < nData; j++) {badAdc->data->dataF[j] = .1*j;}
    badAdc->data->dataUI[0] = 0x7f800000;            /*-- this is INF*/
    badAdc->data->dataUI[1] = 0x7f801000;            /*-- this is NAN*/
    badAdc->dataValid = 1;
    

    /*-- We build two sim1 channel to check the errors messages --*/
    FrAdcData *d1 = FrAdcDataNew(NULL, "D1",sampleRate,nData,-32);
    for(j=0; j < nData; j++) {d1->data->dataF[j] = sin(.01*j);}

    FrSimData *sim1a = FrSimDataNew(NULL, "sim1",sampleRate,nData,32);
    FrSimData *sim1b = FrSimDataNew(NULL, "sim2",sampleRate,nData,32);
    for(j=0; j < nData; j++) {sim1b->data->dataI[j] = j;}

    FrSerData *sms1      = FrSerDataNew(NULL,"sms1",0,"sms data are here", 1.);
    FrMsg *testMsg       = FrMsgAdd(NULL, "Test","Test message",0);
    FrSummary *summaryQ1 = FrSummaryNew(NULL,"Quality_1","main quality"," ",NULL, NULL);
    FrSummary *summaryQ2 = FrSummaryNew(NULL,"Quality_2","aux"," ",NULL, NULL);

    FrEvent *triggerEventA = FrEventNew(NULL,"trigger_A","Binary search","V0:Pr_B1_ACq", 0.,1.,.2,1.e-20,.01,NULL, NULL, 2, "M1", 1.4, "M2", 1.3);
    FrEventAddParam(triggerEventA, "chisquare",2.3);
    printf(" for    event:%s M2=%f\n",triggerEventA->name,FrEventGetParam(triggerEventA,"M2"));

    FrEvent *triggerEventB = FrEventNew(NULL,"trigger_B","Binary search","V0:Pr_B1_ACq", 0.,1.,.2,2.e-20,.01,NULL,NULL, 2,"M1",234.,"M2", 0.123);
    FrEvent *burstEvent = FrEventNew(NULL,"Burst_1","Burst algorithm 1234","V0:Pr_B1_ACp", GTimeS+.2,5.,.3,2.e-20,.02,NULL,NULL,0);

    FrSimEvent *sim1 = FrSimEventNew(NULL,"Sim_Inspiral_1",NULL,NULL,GTimeS+.4,.2,1,1.e-21,0); 
    FrSimEvent *sim2 = FrSimEventNew(NULL,"Sim_Inspiral_2",NULL,NULL,GTimeS+3.,.2,1,1.e-21,0); 
    FrSimEventAddParam(sim2, "M1", 1.4);
    FrSimEventAddParam(sim2, "M2", 1.333);
    printf(" for simEvent:%s M2=%f\n",sim2->name,FrSimEventGetParam(sim2,"M2"));

    FrDetector *detector = FrDetectorNew("L1");

    /*------- add some static data -----------*/
    // FrVect *gData    = FrVectNew(FR_VECT_4R,1,10,1.,"relatif_gain");
    // FrStatData *gain = FrStatDataNew("gain","ADC 1 gain","calibration",
    //                         GTimeS, GTimeS+20, 0, gData, NULL);
    // FrStatDataAdd(frame->detectProc, gain);



    /*-------------------------------------create one filter*/
    // filter = FrFilterNew("f1",1000.,1.,2, 1., 0., 2.34, 2.);
    // FrFilterDump(filter, stdout, 2);
    // FrameAddStatFilter(NULL,frame->detectProc->name, "filter", GTimeS, 1999999999, 0, filter);

                        






    // Saving..
    TString outputFile = "share/ligo/test.gwf";
    FrameFormat::TFrameChain *f = FrameFormat::TFrameChain::Allocate(outputFile, "LIGO", "Trending", 10, GTimeS, 2.);
                              f->AddChannel<double>("MyAdc", 4, {1,2,3,4,5,6,7,8,9,10});
                              f->Print();

    std::cout << "Adding data to file chain." << std::endl;
    for(auto frame : *f) {

        for(int i = 0; i < fastAdc.size(); i++)
            frame->Add(RootCast(fastAdc[i]));

        frame->Add(RootCast(proc));
        for(int i = 0; i < adc.size(); i++)
            frame->Add(RootCast(adc[i]));

        frame->Add(RootCast(table));
        frame->Add(RootCast(badAdc));
        frame->Add(RootCast(d1));
        frame->Add(RootCast(sim1a));
        frame->Add(RootCast(sim1b));
        frame->Add(RootCast(sms1));
        frame->Add(RootCast(testMsg));
        frame->Add(RootCast(summaryQ1));
        frame->Add(RootCast(summaryQ2));
        frame->Add(RootCast(sim1));
        frame->Add(RootCast(sim2));
        frame->Add(RootCast(triggerEventA));
        frame->Add(RootCast(triggerEventB));
        frame->Add(RootCast(burstEvent));
        
        // frame->Add(RootCast(gData)); // get detector proc..
        // frame->Add(RootCast(filter)); // get detector proc..

        frame->SetDataQuality(0x1234);
        frame->Print();
    }

    std::cout << "Applying commit to file chain." << std::endl;
    f->Commit();
    f->Write("share/ligo/test.gwf", "w");
    f->Print();

    delete f;
    f = nullptr;
    
    // Cleaning up..
    for(auto ptr : adc) {
        delete ptr;
    }

    adc.clear();
    delete proc;
    proc = nullptr;

    for(auto ptr : fastAdc) {
        delete ptr;
    }

    fastAdc.clear();
    delete badAdc;
    badAdc = nullptr;
    
    delete d1;
    d1 = nullptr;
    
    delete sim1a;
    sim1a = nullptr;
    delete sim1b;
    sim1b = nullptr;
    
    delete sms1;
    sms1 = nullptr;
    
    delete testMsg;
    testMsg = nullptr;
    
    delete summaryQ1;
    summaryQ1 = nullptr;
    delete summaryQ2;
    summaryQ2 = nullptr;
    
    delete triggerEventA;
    triggerEventA = nullptr;
    delete triggerEventB;
    triggerEventB = nullptr;
    
    delete burstEvent;
    burstEvent = nullptr;
    
    delete sim1;
    sim1 = nullptr;
    delete sim2;
    sim2 = nullptr;

    return(0);
}
