#include <TApplication.h>
#include <Riostream.h>
#include <TLorentzVector.h>
#include <TTree.h>
#include <TLegend.h>
#include <TFile.h>
#include <TKey.h>
#include <TH1.h>
#include <TH2.h>
#include <TH3.h>
#include <TGraph.h>
#include <TCanvas.h>
#include <TStyle.h>
#include <TSystem.h>
#include <TEntryList.h>
#include <TROOT.h>
#include <TFormula.h>
#include <TClass.h>

#include <TFrameFormat.h>
#include <TCli.h>

using namespace std;

int main(int argc, char **argv)
{
        std::vector<TString> ff;
        Bool_t bRead, bTOC, bOverride;
        bool bDump;

        TString options;
        int compression;
        int dataQuality;

        TString select;
        std::vector<int> globalIDs;
        std::vector<double> gps;
        double minDuration, maxDuration;
        
        std::vector<int> channels;
        std::vector<int> groups;
        
        int skip, max;

        TString output;
        bool bFileIntegrityCheck, bFrameIntegrityCheck;
        bool duplicates, bOutputTree;

        int decimate;

        int outputLength;
        int framePerFiles;
        TCli& cmdline = TCli::Instance(&argc, argv, "verbose multithread");
              cmdline.AddMessage("This script is used to verify file checksum..");
              cmdline.AddOption("--compression", "Compression level (0..255)", compression, 0);
              cmdline.AddOption("--skip-file-integrity", "Skip file integrity", bFileIntegrityCheck, true);
              cmdline.AddOption("--skip-frame-integrity", "Skip frame integrity", bFrameIntegrityCheck, true);

              cmdline.AddVariadic(ff, "input.gwf", 1);

        if(!cmdline.Usage() ) return 1;

        // Extract filenames and tags
        for(int i = 0; i < ff.size(); i++) {
               ff[i] = TFileReader::FileName(ff[i]);
        }

        TString fName = ROOT::IOPlus::Helpers::Implode(" ", ff);
        FrameFormat::TFrameChain f(fName.Data());

        FrameFormat::Verbosity verbosity = FrameFormat::Verbosity::kNormal;
        if(gPrint->IsDebugEnabled((int) TCli::Verbosity::kDebug)    ) verbosity = FrameFormat::Verbosity::kAll;
        else if(gPrint->IsDebugEnabled((int) TCli::Verbosity::kAll) ) verbosity = FrameFormat::Verbosity::kDebug;
        else if(gPrint->IsDebugEnabled((int) TCli::Verbosity::kInfo)) verbosity = FrameFormat::Verbosity::kInfo;

        if( TPrint::ErrorIf(!f.IsOpen(), __METHOD_NAME__, "Failed to open `%s`", fName.Data()) )
                return 1;

        int bIntegrity = true;
        if( !bFileIntegrityCheck ) gPrint->Info(__METHOD_NAME__, "File integrity check skipped.");
        else {
                gPrint->Info(__METHOD_NAME__, "Starting file integrity check...");
                if (f.VerifyChecksum() ) {
                        gPrint->Message(__METHOD_NAME__, "Global file integrity check passed.");
                } else {
                        gPrint->Error(__METHOD_NAME__, "Global file integrity check failed.");
                        bIntegrity = false;
                }
        }

        if( !bFrameIntegrityCheck ) gPrint->Info(__METHOD_NAME__, "Frame integrity check skipped.");
        else {
                gPrint->Info(__METHOD_NAME__, "Starting frame integrity check...");
                for(int frameID = 0, nFrames = f.GetNFrames(); frameID < nFrames; frameID++) {
                        FrameFormat::TFrame *frame = f.GetFrame(frameID);
                        if (frame == NULL) {
                                gPrint->Warning(__METHOD_NAME__, "Frame #%d is NULL, skipping.", frameID+1);
                                continue;
                        }

                        if (frame->VerifyChecksum() ) {
                                gPrint->Message(__METHOD_NAME__, "Frame #%0*d integrity check passed.", TString::Itoa(nFrames,10).Length(), frameID+1);
                        } else {
                                gPrint->Error(__METHOD_NAME__, "Frame #%0*d integrity check failed.", TString::Itoa(nFrames,10).Length(), frameID+1);
                                bIntegrity = false;
                        }
                }
        }

        return bIntegrity == 0;
}
