#include <TApplication.h>
#include <Riostream.h>
#include <TLorentzVector.h>
#include <TTree.h>
#include <TLegend.h>
#include <TFile.h>
#include <TKey.h>
#include <TH1.h>
#include <TH2.h>
#include <TH3.h>
#include <TGraph.h>
#include <TCanvas.h>
#include <TStyle.h>
#include <TSystem.h>
#include <TEntryList.h>
#include <TROOT.h>
#include <TFormula.h>
#include <TClass.h>

#include <TFrameFormat.h>
#include <TCli.h>

#include <Framel.h>
using namespace std;

int main(int argc, char **argv)
{
        std::vector<TString> ff;
        Bool_t bRead, bTOC, bOverride;
        bool bDump;

        TString options;
        int compression;
        int dataQuality;

        TString select;
        std::vector<int> globalIDs;
        std::vector<double> gps;
        double minDuration, maxDuration;
        
        std::vector<int> channels;
        std::vector<int> groups;
        
        int skip, max;

        TString output;
        bool bPrint, bHisto;
        bool duplicates, bOutputTree;

        int decimate;

        int newDuration;
        int framePerFiles;
        TCli& cmdline = TCli::Instance(&argc, argv, "verbose multithread");
              cmdline.AddMessage("This script is used to merge files into specific format..");
              cmdline.AddOption("--compression", "Compression level (0..255)", compression, 0);
              cmdline.AddOption("--frames-per-file", "Maximum number of frame per file (default: 1)", framePerFiles, 1);
              cmdline.AddOption("--override", "Override existing files", bOverride);
              cmdline.AddOption("--output-gps-hierarchy", "Output tree structure including truncated GPS time information", bOutputTree);
              cmdline.AddOption("--duration", "Maximum number of frame per file (default: 1)", newDuration, 1);

              cmdline.AddArgument(output, "output.gwf");
              cmdline.AddVariadic(ff, "input.gwf", 1);

        if(!cmdline.Usage() ) return 1;

        if(bOverride) options += "recreate";
        if(bOutputTree) options += " tree";

        // Extract filenames and tags
        TString tags = "*";
        for(int i = 0; i < ff.size(); i++) {

                TString fObject = TFileReader::Obj(ff[i]).Strip(TString::EStripType::kBoth, '/');
                if (fObject.EqualTo("")) continue;

                if (!fObject.EqualTo("") && !tags.EqualTo("*") && !tags.EqualTo(fObject.Data())) {
                        std::cerr << "Ambiguous tag provided. (`" << fObject << "` != `" << tags << "`)" << std::endl;
                        return 1;
                }

                ff[i] = TFileReader::FileName(ff[i]);
                if (!tags.EqualTo("*") && !fObject.EqualTo(tags)) {
                        gPrint->Error(__METHOD_NAME__, "Multiple tagging provided. Please check files and --tags option.");
                        return 1;
                }

                if (!fObject.EqualTo("")) tags = fObject;
        }

        TString fName = ROOT::IOPlus::Helpers::Implode(" ", ff);
        FrameFormat::TFrameChain f(fName.Data(), options, tags);

        FrameFormat::Verbosity verbosity = FrameFormat::Verbosity::kNormal;
        if(gPrint->IsDebugEnabled((int) TCli::Verbosity::kDebug    )) verbosity = FrameFormat::kAll;
        else if(gPrint->IsDebugEnabled((int) TCli::Verbosity::kAll )) verbosity = FrameFormat::kDebug;
        else if(gPrint->IsDebugEnabled((int) TCli::Verbosity::kInfo)) verbosity = FrameFormat::kInfo;
        f.SetVerbosity(verbosity);

        if( TPrint::ErrorIf(!f.IsOpen(), __METHOD_NAME__, "Failed to open `%s`", fName.Data()) )
                return 1;

        FrameFormat::TFrameChain *f0 = f.Reshape(newDuration, options);
        if(f0 == NULL) {
        
                gPrint->Error(__METHOD_NAME__, "Failed to reshape frames.");
                return 1;
        }
        
        f0->Print("toc header");

        return f0->Write(output, options, framePerFiles).size() == 0;
}
