#include <TApplication.h>
#include <Riostream.h>
#include <TLorentzVector.h>
#include <TTree.h>
#include <TLegend.h>
#include <TFile.h>
#include <TKey.h>
#include <TH1.h>
#include <TH2.h>
#include <TH3.h>
#include <TGraph.h>
#include <TCanvas.h>
#include <TStyle.h>
#include <TSystem.h>
#include <TEntryList.h>
#include <TROOT.h>
#include <TFormula.h>
#include <TClass.h>

#include <TFrameFormat.h>
#include <TCli.h>

using namespace std;

int main(int argc, char **argv)
{
        std::vector<TString> ff;
        Bool_t bRead, bTOC, bOverride;
        bool bDump;

        TString options;
        int compression;
        int dataQuality;

        TString select;
        std::vector<int> globalIDs;
        std::vector<double> gps;
        double minDuration, maxDuration;
        
        std::vector<int> channels;
        std::vector<int> groups;
        
        int skip, max;

        TString output;
        bool bPrint, bHisto;
        bool duplicates, bOutputTree;

        int decimate;

        int framePerFiles;
        TCli& cmdline = TCli::Instance(&argc, argv, "verbose multithread");
              cmdline.AddMessage("This script is used to merge files into specific format..");
              cmdline.AddOption("--compression", "Compression level (0..255)", compression, 0);
              cmdline.AddOption("--output-gps-hierarchy", "Output tree structure including truncated GPS time information", bOutputTree);
              cmdline.AddOption("--output-frames-per-file", "Maximum number of frame per file", framePerFiles, 1);
              cmdline.AddOption("--override", "Override existing files", bOverride);

              cmdline.AddArgument(output, "output.gwf");
              cmdline.AddVariadic(ff, "input.gwf", 1);

        if(!cmdline.Usage() ) return 1;

        if(bOverride) options += "recreate";
        if(bOutputTree) options += " tree";

        // Extract filenames and tags
        for(int i = 0; i < ff.size(); i++) {
               ff[i] = TFileReader::FileName(ff[i]);
        }

        TString fName = ROOT::IOPlus::Helpers::Implode(" ", ff);
        FrameFormat::TFrameChain f(fName.Data(), options);

        FrameFormat::Verbosity verbosity = FrameFormat::Verbosity::kNormal;
        if(gPrint->IsDebugEnabled((int) TCli::Verbosity::kDebug    )) verbosity = FrameFormat::kAll;
        else if(gPrint->IsDebugEnabled((int) TCli::Verbosity::kAll )) verbosity = FrameFormat::kDebug;
        else if(gPrint->IsDebugEnabled((int) TCli::Verbosity::kInfo)) verbosity = FrameFormat::kInfo;
        f.SetVerbosity(verbosity);

        if( TPrint::ErrorIf(!f.IsOpen(), __METHOD_NAME__, "Failed to open `%s`", fName.Data()) )
                return 1;

        return f.Write(output, options).size() == 0;
}
